/* Bubba Command Language Header file */

#ifndef _BCL_H
#define _BCL_H

#define LIB_VERSION 0x01

typedef unsigned short SWORD;

#ifndef SMBOARD

#if defined(WIN32) || defined(WINVER)
	#include <windows.h>

#ifdef BCL_LIB
	#define BCLAPI
#else
	#ifdef BCL_DLL
	#define BCLAPI	__declspec( dllexport )
	#else
	#define BCLAPI	__declspec( dllimport )
	#endif
#endif
	#define uchar_t BYTE
	#define ulong_t DWORD
#else

#define BCLAPI

#if defined(WIN32_LEAN_AND_MEAN) || defined(INKERNEL)
	#include "ntddk.h"

        typedef unsigned char uchar_t;
#if (defined(__x86_64__) || defined(__ia64__))
        /* typedef unsigned int ulong_t;  * DWORD is 4 bytes */
#endif
        typedef unsigned long int ulong_t;
        #define strlen Smbstrlen
        #define time Smbtime
        size_t Smbstrlen(const char *string);
        ulong_t Smbtime(ulong_t *Time);

#else
	#include <sys/types.h>
#endif /* WIN32_LEAN_AND_MEAN OR INKERNEL*/

#endif /* WIN32 */

#pragma pack(1)

#endif /* SMBOARD */

/*
**     A general header used for all types of BCL messages.
*/
typedef struct {
   uchar_t magic_cookie;  /* Used to determine whether this is a BCL message */
   uchar_t length;        /* Total number of bytes in the message */
   uchar_t type;          /* Command type: C_GET, C_SET, etc. */
   uchar_t version;       /* Message format version number */
   uchar_t offset;        /* Offset to the first byte after the header */
   uchar_t system;        /* Origin or destination system */
   uchar_t cabinet;       /* Origin or destination cabinet */
   uchar_t chassis;       /* Origin or destination chassis */
   uchar_t frag_offset;   /* Fragment offset - Added 6/11/96 */
   uchar_t frag_id;       /* Fragment id - Added 6/11/96 */
} bcl_header_t;


/*
**     A data structure used to address events and commands to a component level
*/
typedef struct {
   uchar_t group;	  /* Group type: G_BUBBA, G_CHASSIS, etc. */
   uchar_t attr;	  /* Attribute type: A_STATUS, A_POWER_ON, etc. */
   uchar_t unit;	  /* Which unit (where applicable) */
   uchar_t value;		 
}  bcl_cmd_data_t;

/*
**     A BCL C_GET or C_SET command contains the general header and
**     component-level addressing. C_SET also contains data.
*/
typedef struct {
   bcl_header_t header;
   bcl_cmd_data_t command;
} bcl_command_t;


/* 
**     A BCL C_RESPONSE message (in response to a BCL C_GET message) contains
**     the value gotten. Multiple of these may be appended to a single 
**     C_RESPONSE message.
*/
typedef struct {
   uchar_t unit;      /* The unit to which this message pertains */
   uchar_t value;     /* For C_GET, this is the requested data. For C_SET, 
		       * this is the actual data
		       */
} unit_data_t;

/*
**     A BCL C_RESPONSE message has a data portion, defined here.
*/
typedef struct {
   uchar_t group;       /* Group type: G_BUBBA, G_CHASSIS, etc. */
   uchar_t attr;        /* Attribute type: A_STATUS, A_POWER_ON, etc. */
   uchar_t cmd_status;  /* overall command to which this response pertains */
   uchar_t count;       /* The number of (unit,value) pairs that follows */
   unit_data_t value;
} bcl_cmd_response_t;

/*
**     A BCL C_RESPONSE message is made up of the general header and
**     command response data.
*/
typedef struct {
   bcl_header_t header;       
   bcl_cmd_response_t command;
} bcl_response_t;


typedef struct {
   uchar_t chassis_type;
   uchar_t group;
   uchar_t attr;
   uchar_t severity;
   ulong_t time_stamp;
   uchar_t unit;
   uchar_t value;
} bcl_cmd_event_t;

/*
**     A BCL C_EVENT message is made up of the general header and
**     the event data
*/
typedef struct {
   bcl_header_t header;
   bcl_cmd_event_t command;
} bcl_event_t;

/*
**	A BCL Packet message
*/
typedef struct {
   uchar_t start_txt;
   uchar_t seq_number;
} bcl_hdr_t;

typedef struct {
   bcl_hdr_t hdr;
   bcl_header_t header;
   SWORD crc;
} bcl_serial_t;

typedef struct {
   SWORD crc;
} bcl_crc_t;

#ifndef SMBOARD
#pragma pack()
#endif /* SMBOARD */

/* Define sizes and macros */
#define BCL_BUFFER_PAD	20
#define ASSERT_PTR(p) p == NULL? 0:1

#ifndef FIELD_OFFSET
#define FIELD_OFFSET(struct, field) ((int) &(((struct *)(0))->field))
#endif

#ifndef MAKEWORD
#define MAKEWORD(a, b) \
	((SWORD) (((uchar_t) (a)) | ((SWORD) ((uchar_t) (b))) << 8))
#endif /* MAKEWORD */

#ifndef HIBYTE
#define HIBYTE(w)   ((uchar_t) (((SWORD) (w) >> 8) & 0xFF)) 
#endif /* HIBYTE */

#ifndef LOBYTE
#define LOBYTE(w)   ((uchar_t) (w)) 
#endif /* LOBYTE */

#define PACKET_HDR_SIZE  sizeof(bcl_header_t)
#define PACKET_CMD_SIZE  sizeof(bcl_command_t)
#define PACKET_RESPOND_SIZE sizeof(bcl_response_t)
#define PACKET_EVENT_SIZE sizeof(bcl_event_t)
#define PACKET_SERIAL_SIZE sizeof(bcl_serial_t)
#define CRC_SIZE sizeof(bcl_crc_t)

#define MAX_BCL_DATA	(130 + BCL_BUFFER_PAD)
#define MAX_BCL_MESSAGE	(PACKET_CMD_SIZE + MAX_BCL_DATA)
#define MAX_BCL_BUFFER  (PACKET_SERIAL_SIZE + \
						PACKET_HDR_SIZE + \
						CRC_SIZE + \
                        MAX_BCL_DATA)

/********************************CONSTANTS*************************************/

/* Used to identify BCL version */
#define BCL_VERSION	0x01
#define MAGIC_COOKIE	0xBC
#define DATA_OFFSET	0x0A

/* Command Types */
#define C_GET 		0x47
#define C_SET 		0x53
#define C_RESPONSE 	0x52
#define BCL_ACK 	0x41
#define BCL_EVENT 	0x45

/* Command status */
#define BCL_FAIL	0x00
#define BCL_SUCCESS	0x01

/* BCL Start and End Text */
#define BCL_STX		0x02

/* Chassis types */
/* OTHER = 0x01, UNKNOWN=0x02 */
#define CH_DESKSIDE	0x03
#define CH_RM_DISK	0x04
#define CH_RM_UPS	0x05
#define CH_SRV_MGMT	0x06
#define CH_RM_NODE	0x07

// Added 7/3/96
#if LIB_VERSION > 1
#define CH_RM_BYNET 0x08
#endif

/* Group Types */
#define G_UNDEFINED		0x00
#define G_COOLING  		0x0A
#define G_STORAGE 		0x1E
#define G_BATTERY 		0x32
#define G_PWR_SUPPLY		0x46
#define G_NODE_CHASSIS		0x5A	
#define G_CHASSIS 		0x78
#define G_BUBBA  		0xA0

/* Wildcards for chassis */
#define ALL_CHASSIS		0xFF
#define ALL_SRV_MGMT_CHASSIS	0x80
#define ALL_NODE_CHASSIS	0x81
#define ALL_UPS_CHASSIS		0x82
#define ALL_DISK_CHASSIS	0x83

/* Wildcards for cabinets */
#define ALL_AVAIL_CABINETS	0xFF

/* Unit IDs */
#define BCL_ALL_UNITS	0xF0
#define BCL_UNIT_UNUSED	0xFF

/* Valid Values  - Enable enum*/
#define OTHER		0x01
#define UNKNOWN		0x02
#define ENABLE		0x01
#define DISABLE		0x00

/* Valid Values - Status enum */
/* OTHER=0x01 UNKNOWN=0x02 */
#define ST_NORMAL	0x03
#define ST_WARNING	0x04
#define ST_CRITICAL	0x05
#define ST_MISSING	0x06
#define ST_OFF		0x07
#define ST_PASS		0x01
#define ST_FAIL		0x00

/* Event Severities */
/* Ascii Rep. : 0x49('I') 0x4F('O') 0x4E('N') 0x43('C') 0x46('F') 0x4D('M') */
#define E_INFO			0x49	
#define E_OK			0x4F
#define E_NON_CRITICAL		0x4E
#ifdef E_CRITICAL
#undef E_CRITICAL
#endif
#define E_CRITICAL		0x43
#define E_NON_RECOVERABLE 	0x46
#define E_MONITOR	 	0x4D

/* Battery Inputs */
/* OTHER=0x01 UNKNOWN=0x02 */
#define B_AC_OK		0x03
#define B_AC_FAIL	0x04

/* Packet elements */
#define A_PKT_MAGIC		FIELD_OFFSET(bcl_header_t, magic_cookie)
#define A_PKT_LENGTH		FIELD_OFFSET(bcl_header_t, length)
#define A_PKT_TYPE		FIELD_OFFSET(bcl_header_t, type)
#define A_PKT_VERSION		FIELD_OFFSET(bcl_header_t, version)
#define A_PKT_OFFSET		FIELD_OFFSET(bcl_header_t, offset)
#define A_PKT_SYSTEM		FIELD_OFFSET(bcl_header_t, system)
#define A_PKT_CABINET		FIELD_OFFSET(bcl_header_t, cabinet)
#define A_PKT_CHASSIS		FIELD_OFFSET(bcl_header_t, chassis)

#define A_PKT_GROUP		FIELD_OFFSET(bcl_command_t, command.group)
#define A_PKT_ATTR		FIELD_OFFSET(bcl_command_t, command.attr)
#define A_PKT_UNITID		FIELD_OFFSET(bcl_command_t, command.unit)
#define A_PKT_VALUE		FIELD_OFFSET(bcl_command_t, command.value)

#define A_PKT_EVENT_CH_TYPE	FIELD_OFFSET(bcl_event_t, command.chassis_type)
#define A_PKT_EVENT_GROUP	FIELD_OFFSET(bcl_event_t, command.group)
#define A_PKT_EVENT_ATTR	FIELD_OFFSET(bcl_event_t, command.attr)
#define A_PKT_EVENT_SEVERITY	FIELD_OFFSET(bcl_event_t, command.severity)
#define A_PKT_EVENT_TIMESTAMP	FIELD_OFFSET(bcl_event_t, command.time_stamp)
#define A_PKT_EVENT_UNITID	FIELD_OFFSET(bcl_event_t, command.unit)
#define A_PKT_EVENT_VALUE	FIELD_OFFSET(bcl_event_t, command.value)


/* OutofBand destination types */
/* OTHER=0x01 UNKNOWN=0x02 */
#define OB_IXO_PAGER	0x03
#define OB_NUM_PAGER	0x04
#define OB_CONSOLE	0x05
#define OB_TTY		0x06

/* OutofBand Connection types */
/* OTHER=0x01 UNKNOWN=0x02 */
#define CONN_INTERNAL_MODEM	0x03
#define CONN_EXTERNAL_MODEM	0x04
#define CONN_RS232		0x05
#define CONN_NONE		0x06

/* OutofBand Connection state */
/* OTHER=0x01 UNKNOWN=0x02 */
#define ST_IDLE			0x03
#define ST_PASS_THRU		0x04
#define ST_COMMAND		0x05
#define ST_CONSOLE_REDIR	0x06

/* OutofBand Connection Mode */
/* OTHER=0x01 UNKNOWN=0x02 */
#define MODE_SERVER_MGMT	0x03
#define MODE_GEN_PURPOSE	0x04

/* Fan Speeds */
/* UNKNOWN=0x02 */
#define SP_NORMAL	0x03
#define SP_FAST		0x04
#define SP_MAXIMUM	0x05

/* Modem Speed */
/* OTHER=0x01 UNKNOWN=0x02 */
#define COMSPEED_9600	0x03
#define COMSPEED_14400	0x04
#define COMSPEED_19200	0x05
#define COMSPEED_28800	0x06
#define COMSPEED_38400	0x07
#define COMSPEED_57600	0x08
#define COMSPEED_115200	0x09

/* SMB  States */
/* OTHER=0x01 UNKNOWN=0x02 */
#define SMB_ALL_OK		0x03
#define SMB_FAILURE		0x04
#define SMB_DRIVER_FAILURE	0x05
#define SMB_AGENT_FAILURE	0x06

/* General */
#if !defined(FALSE) && !defined(TRUE)
#define FALSE		0x00
#define TRUE		0x01
#endif

/*********************************ATTRIBUTES***********************************/

/* General attributes - for any groups */
#define A_STATUS 		0x01
#define A_POWER_ON 		0x02
#define A_POWER_OFF 	0x03

/* Cooling Attributes */

/* Storage Attributes */

/* Power Supply */
/* Default attributes: general attributes */

/* Battery */
#define A_BATTERY_STATE		G_BATTERY + 0
#define A_BATTERY_INPUT		G_BATTERY + 1

/* Node Attributes */
#define A_NODE_SHUTDOWN		G_NODE_CHASSIS + 0
#define A_NODE_REBOOT		G_NODE_CHASSIS + 1
#define A_NODE_NMI			G_NODE_CHASSIS + 2
#define A_NODE_LAST_POSTCODE	G_NODE_CHASSIS + 3
#define A_NODE_AGENT_DOWN	G_NODE_CHASSIS + 4

/* Added on 5/28/96 */
#define A_NODE_CHASSEVENT_LOGGED	G_NODE_CHASSIS + 5
#define	A_NODE_DOWN_WAIT			G_NODE_CHASSIS + 6
#define A_NODE_LASTEVENT_LOGGED		G_NODE_CHASSIS + 7
#define A_NODE_SINGLE_BYTE			G_NODE_CHASSIS + 8

/* Chassis Attributes */
#define A_CH_ASSETTAG 		G_CHASSIS + 0
#define A_CH_CHASSISTAG		G_CHASSIS + 1
#define A_CH_MODEL_NO		G_CHASSIS + 2
#define A_CH_SERIALNO 		G_CHASSIS + 3
#define A_CH_RESET			G_CHASSIS + 4
#define A_CH_POWER_CYCLE	G_CHASSIS + 5
#define A_CH_TYPE			G_CHASSIS + 6
#define A_CH_PWR_STATE		G_CHASSIS + 7
#define A_CH_THERMAL_STATE	G_CHASSIS + 8
#define A_CH_SYSNO			G_CHASSIS + 9
#define A_CH_CABNO			G_CHASSIS + 10
#define A_CH_LOCATION		G_CHASSIS + 11
#define A_CH_HEIGHT			G_CHASSIS + 12
#define A_CH_DESC			G_CHASSIS + 13
#define A_CH_POWER_ON_DELAY G_CHASSIS + 14
#define A_CH_POWER_OFF_DELAY G_CHASSIS + 15

/* SMB Attributes */
#define A_SMB_SERIAL_NO			G_BUBBA + 0
#define A_SMB_FW_REVISION		G_BUBBA + 1
#define A_SMB_ASSEMBLY_NO		G_BUBBA + 2
#define A_SMB_RESET				G_BUBBA + 3
#define A_SMB_FLASH_BEGIN		G_BUBBA + 4
#define A_SMB_FLASH_END			G_BUBBA + 5
#define A_SMB_FLASH_ROLLBACK	G_BUBBA + 6
#define A_SMB_FLASH_COMMIT		G_BUBBA + 7
#define A_SMB_HB_INTERVAL		G_BUBBA + 8
#define A_SMB_OB_ENABLE			G_BUBBA + 9
#define A_SMB_OB_PHONENO		G_BUBBA + 10
#define A_SMB_OB_DESTTYPE		G_BUBBA + 11
#define A_SMB_OB_CONNTYPE		G_BUBBA + 12
#define A_SMB_DIAL_PREFIX		G_BUBBA + 13
#define A_SMB_CONN_STATE		G_BUBBA + 14
#define A_SMB_CONN_MODE			G_BUBBA + 15
#define A_SMB_BAUD_RATE			G_BUBBA + 16
#define A_SMB_MODEM_INIT_STR	G_BUBBA + 17
#define A_SMB_MODEM_RESET		G_BUBBA + 18
#define A_SMB_INB_PHONENO		G_BUBBA + 19
#define A_SMB_INB_ENABLE		G_BUBBA + 20
#define A_SMB_USER_PWD			G_BUBBA + 21
#define A_SMB_PASS_THRU			G_BUBBA + 22
#define A_SMB_CONSOLE_REDIR		G_BUBBA + 23
#define A_SMB_SEND_EVENT		G_BUBBA + 24
#define A_SMB_PAUSE_PERIOD		G_BUBBA + 25
#define A_SMB_IXO_PASSWORD		G_BUBBA + 26
#define A_SMB_UPDATE_COMPLETE	G_BUBBA + 27
#define A_SMB_UPDATE_ABORT		G_BUBBA + 28
#define A_SMB_UPDATE_REQUEST	G_BUBBA + 29
#define A_SMB_FLASH_DATA		G_BUBBA + 30
#define A_SMB_PAGER_ID			G_BUBBA + 31
#define A_SMB_STATUS_CHANGE		G_BUBBA + 32
#define A_SMB_FLASH_CORRUPTED	G_BUBBA + 33
#define A_SMB_HFWEVENT_LOGGED	G_BUBBA + 34
#define A_SMB_FLASH_WRITE_OVER_LIMIT G_BUBBA + 35
#define A_SMB_FLASH_SOURCE		G_BUBBA + 36
#define A_SMB_FLASH_UPDATE		G_BUBBA + 37

/****************************P R O T O T Y P E S******************************/
#ifndef SMBOARD

#ifdef __cplusplus
extern "C" {
#endif

BCLAPI uchar_t BclAddressPacket(void *cmd, uchar_t system, uchar_t cabinet, 
							uchar_t chassis);
BCLAPI uchar_t *BclGetPacketMember(void *cmd, uchar_t name);
BCLAPI SWORD BclGetVersion();
BCLAPI int BclVerifyPacket(void *cmd, ulong_t packetSize);
BCLAPI void BclInitMemory(void *cmd, int c, size_t n);
BCLAPI uchar_t BclSetPacketSize(void *cmd, uchar_t size);
BCLAPI uchar_t BclBuildPacket(void *cmd, uchar_t seq);
BCLAPI uchar_t BclBuildGetCmd(bcl_command_t *cmd, uchar_t group, uchar_t attrib, 
							uchar_t unit);
BCLAPI uchar_t BclBuildSetCmd(bcl_command_t *cmd, uchar_t group, uchar_t attrib, 
						uchar_t unit, uchar_t *value);
BCLAPI uchar_t BclBuildSetCmdEx(bcl_command_t *cmd, uchar_t group, uchar_t attrib, 
						uchar_t unit, uchar_t *value, ulong_t length);
BCLAPI uchar_t BclBuildResponse(bcl_response_t *cmd, uchar_t group, uchar_t attrib, 
			uchar_t respStatus, uchar_t count, 
			unit_data_t *unitStatus, uchar_t size);
BCLAPI uchar_t BclBuildACK(bcl_serial_t *cmd);
BCLAPI uchar_t BclBuildEvent(bcl_event_t *cmd, uchar_t group, uchar_t attrib,
			uchar_t unit, uchar_t severity, uchar_t *value);
BCLAPI uchar_t BclBuildCmd(bcl_command_t *cmd, uchar_t cmdtype, uchar_t group,
							uchar_t attrib,	uchar_t unit, uchar_t *data, 
							ulong_t length);

// Internal Use ONLY 
extern uchar_t SetGroup(void *cmd, uchar_t group);
extern uchar_t SetAttrib(void *cmd, uchar_t group, uchar_t attrib);
extern uchar_t SetUnitValue(bcl_command_t *cmd, uchar_t group, uchar_t attrib, 
		uchar_t *value);
extern uchar_t SetUnitValueEx(bcl_command_t *cmd, uchar_t group, uchar_t attrib, 
		uchar_t *value, ulong_t length);
extern uchar_t SetStatus(bcl_response_t *cmd, uchar_t unit);
extern uchar_t SetResponseValue(bcl_response_t *cmd, uchar_t count, 
			 unit_data_t *value, uchar_t size);
extern uchar_t SetSeverity(bcl_event_t *cmd, uchar_t severity);
extern uchar_t SetUnit(void *cmd, uchar_t unit);
extern uchar_t SetEventValue(bcl_event_t *cmd, uchar_t group, uchar_t attr, 
	              uchar_t *value);
extern SWORD Crc16(uchar_t *Buffer, int Length);

#ifdef __cplusplus
}
#endif

/***************************
 * Server Management Board *
 ***************************/
#define BCL_BUILDGET_SMBCMD(cmd, attrib, unit) \
BclBuildGetCmd(cmd, G_BUBBA, attrib, unit)

#define BCL_BUILDSET_SMBCMD(cmd, attrib, unit, value) \
BclBuildSetCmd(cmd, G_BUBBA, attrib, unit, value)

#define BCL_BUILDRESPONSE_GETSMB(cmd, attrib, cmd_status, cnt, unit, size) \
BclBuildResponse(cmd, G_BUBBA, attrib, cmd_status, cnt, unit, size)

#define BCL_BUILDRESPONSE_SETSMB(cmd, attrib, cmd_status, cnt, unit, size) \
BclBuildResponse(cmd, G_BUBBA, attrib, cmd_status, cnt, unit, size)


/***************************
 * Chassis                 *
 ***************************/
#define BCL_BUILDGET_CHASSISCMD(cmd, attrib, unit) \
BclBuildGetCmd(cmd, G_CHASSIS, attrib, unit)

#define BCL_BUILDSET_CHASSISCMD(cmd, attrib, unit, value) \
BclBuildSetCmd(cmd, G_CHASSIS, attrib, unit, value)

#define BCL_BUILDRESPONSE_GETCHASSIS(cmd, attrib, cmd_status, cnt, unit, size) \
BclBuildResponse(cmd, G_CHASSIS, attrib, cmd_status, cnt, unit, size)

#define BCL_BUILDRESPONSE_SETCHASSIS(cmd, attrib, cmd_status, cnt, unit, size) \
BclBuildResponse(cmd, G_CHASSIS, attrib, cmd_status, cnt, unit, size)


/***************************
 * Node                    *
 ***************************/
#define BCL_BUILDGET_NODECMD(cmd, attrib, unit) \
BclBuildGetCmd(cmd, G_NODE_CHASSIS, attrib, unit)

#define BCL_BUILDSET_NODECMD(cmd, attrib, unit, value) \
BclBuildSetCmd(cmd, G_NODE_CHASSIS, attrib, unit, value)

#define BCL_BUILDRESPONSE_GETNODE(cmd, attrib, cmd_status, cnt, unit, size) \
BclBuildResponse(cmd, G_NODE_CHASSIS, attrib, cmd_status, cnt, unit, size)

#define BCL_BUILDRESPONSE_SETNODE(cmd, attrib, cmd_status, cnt, unit, size) \
BclBuildResponse(cmd, G_NODE_CHASSIS, attrib, cmd_status, cnt, unit, size)


/***************************
 * Battery (UPS)           *
 ***************************/
#define BCL_BUILDGET_UPSCMD(cmd, attrib, unit) \
BclBuildGetCmd(cmd, G_BATTERY, attrib, unit)

#define BCL_BUILDSET_UPSCMD(cmd, attrib, unit, value) \
BclBuildSetCmd(cmd, G_BATTERY, attrib, unit, value)

#define BCL_BUILDRESPONSE_GETUPS(cmd, attrib, cmd_status, cnt, unit, size) \
BclBuildResponse(cmd, G_BATTERY, attrib, cmd_status, cnt, unit, size)

#define BCL_BUILDRESPONSE_SETUPS(cmd, attrib, cmd_status, cnt, unit, size) \
BclBuildResponse(cmd, G_BATTERY, attrib, cmd_status, cnt, unit, size)


/***************************
 * Power Supply            *
 ***************************/
#define BCL_BUILDGET_PWRCMD(cmd, attrib, unit) \
BclBuildGetCmd(cmd, G_PWR_SUPPLY, attrib, unit)

#define BCL_BUILDSET_PWRCMD(cmd, attrib, unit, value) \
BclBuildSetCmd(cmd, G_PWR_SUPPLY, attrib, unit, value)

#define BCL_BUILDRESPONSE_GETPWR(cmd, attrib, cmd_status, cnt, unit, size) \
BclBuildResponse(cmd, G_PWR_SUPPLY, attrib, cmd_status, cnt, unit, size)

#define BCL_BUILDRESPONSE_SETPWR(cmd, attrib, cmd_status, cnt, unit, size) \
BclBuildResponse(cmd, G_PWR_SUPPLY, attrib, cmd_status, cnt, unit, size)


/***************************
 * Cooling                 *
 ***************************/
#define BCL_BUILDGET_FANCMD(cmd, attrib, unit) \
BclBuildGetCmd(cmd, G_COOLING, attrib, unit)

#define BCL_BUILDSET_FANCMD(cmd, attrib, unit, value) \
BclBuildSetCmd(cmd, G_COOLING, attrib, unit, value)

#define BCL_BUILDRESPONSE_GETFAN(cmd, attrib, cmd_status, cnt, unit, size) \
BclBuildResponse(cmd, G_COOLING, attrib, cmd_status, cnt, unit, size)

#define BCL_BUILDRESPONSE_SETFAN(cmd, attrib, cmd_status, cnt, unit, size) \
BclBuildResponse(cmd, G_COOLING, attrib, cmd_status, cnt, unit, size)


/***************************
 * Miscellaneous           *
 ***************************/
#define BCL_BUILDGET_STATUS(cmd, group, unit) \
BclBuildGetCmd(cmd, group, A_STATUS, unit)

#define BCL_BUILDSET_POWERON(cmd, group, unit) \
BclBuildSetCmd(cmd, group, A_POWER_ON, unit, BCL_UNIT_UNUSED)

#define BCL_BUILDSET_POWEROFF(cmd, group, unit) \
BclBuildSetCmd(cmd, group, A_POWER_OFF, unit, BCL_UNIT_UNUSED)

#endif /* SMBOARD */

#endif  /* _BCL_H */
