
/*M*
//            Copyright (c) 1995 Intel Corporation.  
//
//  Purpose:    This file which contains all the i2capi.dll related constants
//              
//  Contents:
//	GetI2CAccessStatus 
//	I2CReadSlaveDevice
//	I2CMasterWrite
//	I2CMasterWriteRead
//	I2CEnableResponseData
//	I2CGetResponse
//	I2CGetSingleByteEvent

 *  This program is free software; you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License as published by the
 *  Free Software Foundation; either version 2 of the License, or (at your
 *  option) any later version.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 *  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 *  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 *  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 *  TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 *  USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  675 Mass Ave, Cambridge, MA 02139, USA.  
*M*/

/******************************************************************************\
*
*                                 I2CAPI.H
*
\******************************************************************************/

#ifndef _I2CAPI_H
#define _I2CAPI_H

////////////////////////////////////////////////////////////////////////////////
//  I2CLogicalBus
////////////////////////////////////////////////////////////////////////////////

/*D*
//  Name:       I2CLogicalBus
//  Purpose:    defines the logical I2C buses
//  Context:    Used in functions for getting the I2C related information
//  Fields:     
//	I2C_BUS_RAW0 - Raw data(no protocol module) from Primary I2C bus
//	I2C_BUS_RAW1 - Raw data(no protocol module) from Secondary I2C bus
//	I2C_BUS_CABINET0 - Cabinet protocol data from Primary I2C bus
//	I2C_BUS_CABINET1 - Cabinet protocol data from Secondary I2C bus
//	I2C_BUS_SMB - System Management protocol data from Primary I2C bus
*D*/

typedef enum {

	I2C_BUS_RAW0,
	I2C_BUS_RAW1,
	I2C_CABINET0,
	I2C_CABINET1,
	I2C_SMB

} I2CLogicalBus; 

////////////////////////////////////////////////////////////////////////////////
//  I2COptions
////////////////////////////////////////////////////////////////////////////////

/*D*
//  Name:       I2COptions
//  Purpose:    defines the I2C Read options
//  Context:    Used in functions for getting the I2C related information
//  Fields:     
//	I2C_NO_WAIT - I2C Reads do not wait for data
//	I2C_WAIT_FOR_DATA - I2C Reads block waiting for data
*D*/

typedef enum {

	I2C_NO_WAIT = 0,
	I2C_WAIT_FOR_DATA

} I2COptions; 

////////////////////////////////////////////////////////////////////////////////
//  I2CAccessStatus
////////////////////////////////////////////////////////////////////////////////

/*D*
//  Name:       I2CAccessStatus
//  Purpose:    defines the return values for I2C access
//  Context:    Used in functions for getting the I2C related information
//  Fields:     
//	I2c_AccessOK - command completed successfully.
//	I2c_InvalidBus - bad or incorrect bus number field.
//	I2c_InvalidLength - BytesToRead was not greater 0 and less than 1025.
//	I2c_BadAddress - no I2C device responded to the supplied address
//	I2c_BusHung - the I2C driver never successfully arbitrated for the I2C
//			bus.  Probably a hardware problem.
//	I2c_NoData - when using the I2C_NO_WAIT option, this status indicates
//	the requested data is not currently available.  This status should
//	only be returned when using I2C_NO_WAIT.
//	I2c_NoEvents - No Single byte event messages
//	I2c_Unknown - An unexpect error occurred
*D*/

typedef enum {
	
	I2c_AccessOK,
	I2c_InvalidBus,
	I2c_InvalidLength,
	I2c_BadAddress,
	I2c_BusHung,
	I2c_BusBusy,
	I2c_NoData,
	I2c_NoEvents,
	I2c_InsufficientResources,
	I2c_Unknown

} I2CAccessStatus; 

// Number of times to try an ioctl if it fails with busy status
#define	I2C_API_TRIES	7


/******************************************************************************\
*                              FUNCTION PROTOTYPES
\******************************************************************************/


#ifdef __cplusplus
extern "C" {
#endif


#include "smidebug.h"
////////////////////////////////////////////////////////////////////////////////
// I2CEnableDebug
////////////////////////////////////////////////////////////////////////////////

/*F*
//  Name:       I2CEnableDebug
//  Purpose:    Turn on some amount of debugging
//  Context:	called at anytime to enable or disable I2C library debugging
//  Returns:    none
//  Parameters:
//      DebugFunction - pointer to user supplied debug function
//      level - debug level, see smidebug.h
//  Notes:      none
*F*/

void
I2CEnableDebug(
	PDEBUG_ROUTINE Function,
	SMIDebugLevels Level
	);

////////////////////////////////////////////////////////////////////////////////
// GetI2CAccessStatus
////////////////////////////////////////////////////////////////////////////////

/*F*
//  Name:       GetI2CAccessStatus
//  Purpose:    To get the last error code
//  Context:    Call after an IOCTL fails
//  Returns:    I2CAccessStatus
//  Parameters: none
//  Notes:      none
*F*/

I2CAccessStatus
GetI2CAccessStatus (void);

////////////////////////////////////////////////////////////////////////////////
// I2CReadSlaveDevice
////////////////////////////////////////////////////////////////////////////////

/*F*
//
//  Name:       I2CReadSlaveDevice
//
//  Purpose:    This function reads data from an I2C slave device using the
//              Master receive I2C transfer mode.
//
//  Context:    Used by Upper level agents (sis modules) to get I2C information
//  Returns:	I2CAccessStatus
//      BytesActuallyWritten contains the number of bytes successfully read
//              from the target device.
//  Parameters:
//      LogicalBus      specifies which baseboard I2C controller and bus
//              protocol to use.
//      Address specifies the I2C bus address of the slave device to be read.
//      BytesToRead     contains the expected number of bytes to read from
//              the slave device.  The ReadBuffer must be at least this many
//              bytes long.  Currently, this field must be greater than 0
//              and less than 1025.
//      ReadBuffer      is a pointer to the receiving buffer for the data from
//              the slave device.  A maximum of 1k (1024 bytes) can be read
//              with this function.
//      BytesActuallyRead       is a pointer to a location that will be updated
//              with the number of bytes actually read into the ReadBuffer.
//              On failures, this field will contain zero.
//  Notes:      none
//
*F*/

I2CAccessStatus
I2CReadSlaveDevice(
	I2CLogicalBus	LogicalBus,
	unsigned long	Address,
	unsigned long	BytesToRead,
	unsigned char 	*ReadBuffer,
	unsigned long	*BytesActuallyRead
	);

////////////////////////////////////////////////////////////////////////////////
// I2CMasterWrite
////////////////////////////////////////////////////////////////////////////////

/*F*
//  Name:       I2CMasterWrite
//
//  Purpose:    This function writes data to an I2C device using the Master
//              transmit I2C transfer mode.
//  Context:    Used by Upper level agents (sis modules) to write I2C information
//  Returns:
//      I2CAccessStatus
//      BytesActuallyWritten contains the number of bytes successfully written to the target device.
//  Parameters:
//      LogicalBus      specifies which baseboard I2C controller and bus
//              protocol to use
//      Address specifies the I2C bus address of the target device for the write data.
//      BytesToWrite    contains the expected number of bytes to send to the
//              target device.  The WriteBuffer must be at least this many
//              bytes long.  Currently, this field must be greater than 0
//              and less than 1025.
//      WriteBuffer     is a pointer to the buffer containing the  data for
//              the target I2C device.  A maximum of 1k (1024 bytes) can be
//              written with this function.
//      BytesActuallyWritten    is a pointer to a location that will be updated
//              with the number of bytes successfully written to the target
//              device.  On failures, this field will contain zero.
//  Notes:      none
*F*/

I2CAccessStatus
I2CMasterWrite(
	I2CLogicalBus	LogicalBus,
	unsigned long	Address,
	unsigned long	BytesToWrite,
	unsigned char 	*WriteBuffer,
	unsigned long	*BytesActuallyWritten
	);

////////////////////////////////////////////////////////////////////////////////
// I2CMasterWriteRead
////////////////////////////////////////////////////////////////////////////////

/*F*
//  Name:       I2CMasterWriteRead
//  Purpose:    This function writes data to an I2C device using the
//              Master transmit I2C transfer mode and then reads the same I2C
//              device in the Master receive mode.  This routine is intended
//              for slave devices that have data bits whose values depend on
//              what is written to the port.  For example, bits 3 thru 6 are
//              fan status unless a one is written to bit 7 in which case they
//              are processor status bits.
//  Context:    Used by Upper level agents (sis modules) to write I2C informatio
n
//  Returns:
//      I2CAccessStatus
//      BytesActuallyWritten contains the number of bytes successfully written
//              to the target device.
//  Parameters:
//      LogicalBus      specifies which baseboard I2C controller and bus
//              protocol to use
//      Address specifies the I2C bus address of the target device for the write data.
//      BytesToWrite    contains the expected number of bytes to send to the
//              target device.  The WriteBuffer must be at least this many
//              bytes long.  Currently, this field must be greater than 0
//              and less than 1025.
//      WriteBuffer     is a pointer to the buffer containing the  data for
//              the target I2C device.  A maximum of 1k (1024 bytes) can be
//              written with this function.
//      BytesToRead     contains the expected number of bytes to read from
//              the slave device.  The ReadBuffer must be at least this many
//              bytes long.  Currently, this field must be greater than 0
//              and less than 1025.
//      ReadBuffer      is a pointer to the receiving buffer for the data from
//              the slave device.  A maximum of 1k (1024 bytes) can be read
//              with this function.
//      BytesActuallyRead       is a pointer to a location that will be updated
//              with the number of bytes actually read into the ReadBuffer.
//              On failures, this field will contain zero.
//  Notes:      none
*F*/

I2CAccessStatus
I2CMasterWriteRead(
	I2CLogicalBus	LogicalBus,
	unsigned long	Address,
	unsigned long	BytesToWrite,
	unsigned char 	*WriteBuffer,
	unsigned long	BytesToRead,
	unsigned char 	*ReadBuffer,
	unsigned long	*BytesActuallyRead
	);

////////////////////////////////////////////////////////////////////////////////
// I2CEnableResponseData
////////////////////////////////////////////////////////////////////////////////

/*F*
//  Name:       I2CEnableResponseData
//  Purpose:    This function tells the I2C driver to give response data
//              received from an I2C device to the process
//  Context:    Used by Upper level agents (sis modules) to get I2C information
//  Returns:
//      I2CAccessStatus
//  Parameters:
//      LogicalBus      specifies which baseboard I2C controller and bus
//              protocol to use
//      Options specifies option bits that can modify the behavior of the
//              function.  At this time, only two options are defined:
//      	IC2_NO_WAIT - do not block waiting for I2C response data
//      	IC2_WAIT_FOR_DATA - block until I2C response data is received
//	BufferSize specifies the buffer size allocated by the driver for
//		response data. A buffer size of zero directs the driver to
//		use the default size. The default buffer size is 1024 bytes.
//  Notes:      none
*F*/

I2CAccessStatus
I2CEnableResponseData(
        I2CLogicalBus   LogicalBus,
        I2COptions	Options,
	unsigned long	BufferSize
        );

////////////////////////////////////////////////////////////////////////////////
// I2CGetResponse
////////////////////////////////////////////////////////////////////////////////

/*F*
//  Name:       I2CGetResponse
//  Purpose:    This function returns response data received from an I2C device
//              Normally, a call to this function would be preceded by a call
//              to I2CMasterWrite to send a command to an I2C master device.
//              Unless, the IC2_NO_WAIT option is used, the call will block
//              waiting until response data is received.
//  Context:    Used by Upper level agents (sis modules) to get I2C information
//  Returns:
//      I2CAccessStatus
//      BytesActuallyRead contains the number of bytes read from the target device.
//  Parameters:
//      LogicalBus      specifies which baseboard I2C controller and bus
//              protocol to use
//      BytesToRead     contains the number of bytes to expect in the response message.
//              The ReadBuffer must be at least this many bytes long.
*F*/

I2CAccessStatus
I2CGetResponse(
	I2CLogicalBus	LogicalBus,
	unsigned long	BytesToRead,
	unsigned char 	*ReadBuffer,
	unsigned long	*BytesActuallyRead
	);

////////////////////////////////////////////////////////////////////////////////
// I2CGetSingleByteEvent
////////////////////////////////////////////////////////////////////////////////

I2CAccessStatus
I2CGetSingleByteEvent (
	unsigned long	EventType
	);



#ifdef __cplusplus
}
#endif

#endif // _I2CAPI_H
