#ifndef _I2CDRV_H
#define _I2CDRV_H

#ifdef UWSVR4
#define DEVICE_I2CA		"/dev/instruments/i2cA"
#define DEVICE_I2CB		"/dev/instruments/i2cB"
#define DEVICE_CABINETA		"/dev/instruments/cabinetA"
#define DEVICE_CABINETB		"/dev/instruments/cabinetB"
#define DEVICE_SMB		"/dev/instruments/smb"
#else
#define DEVICE_I2CA		"\\\\.\\I2CA"
#define DEVICE_I2CB		"\\\\.\\I2CB"
#define DEVICE_CABINETA	"\\\\.\\CABINETA"
#define DEVICE_CABINETB	"\\\\.\\CABINETB"
#define DEVICE_SMB		"\\\\.\\SMB"
#endif

//
// Device Type
//


#define FILE_DEVICE_I2C	0x00008002

//
// I/O Control codes
//

#define IOCTL_I2C_BASE			0x0800


#define IOCTL_I2C_READ_SLAVE_DEVICE	(ULONG)CTL_CODE(FILE_DEVICE_I2C, IOCTL_I2C_BASE + 1, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_I2C_MASTER_WRITE		(ULONG)CTL_CODE(FILE_DEVICE_I2C, IOCTL_I2C_BASE + 2, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_I2C_WRITE_READ		(ULONG)CTL_CODE(FILE_DEVICE_I2C, IOCTL_I2C_BASE + 3, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_I2C_WANT_RESPONSE_DATA	(ULONG)CTL_CODE(FILE_DEVICE_I2C, IOCTL_I2C_BASE + 4, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_I2C_CHANGE_PARAMETERS	(ULONG)CTL_CODE(FILE_DEVICE_I2C, IOCTL_I2C_BASE + 5, METHOD_BUFFERED, FILE_ANY_ACCESS)



//
// Command: I2CReadSlaveDevice
// 
// Reads data from an I2C slave device in Master Receive mode.
// Uses fields of struct smi as follows:
// lpvInBuffer - pointer to struct of type i2cReadRequest.
// cbInBuffer - size of struct i2cReadRequest
// lpvOutBuffer - pointer to buffer area for received data
// cbOutBuffer - size of the buffer area (maximum number of bytes to read)
// lpcbBytesReturned - pointer to location that will contains actual bytes read.
//	i2cAddress - the address of the I2C slave device to read. 
//
//

#pragma pack(1)
typedef struct {
	ULONG		I2CAddress;
} I2C_READ_REQUEST, *PI2C_READ_REQUEST;
#pragma pack()

//
// Command: I2CMasterWrite
// 
// Writes data to an I2C slave device in Master Transmit mode.
// Uses fields of nt2unixioctl as follows:
// lpvInBuffer - pointer to struct of type i2cWriteRequest.
// cbInBuffer - size of struct i2cWriteRequest plus the data size
// lpvOutBuffer - zero (not used)
// cbOutBuffer - zero
// lpcbBytesReturned - zero.
//	i2cAddress - the address of the I2C device to write the data to
//	i2cData - first byte of the data to write.  If writing more than
//		one byte, the remaining bytes to write immediately follow
//		this byte in the input buffer.
//

#pragma pack(1)
typedef struct {
	ULONG	I2CAddress;
	UCHAR	I2CData;
} I2C_WRITE_REQUEST, *PI2C_WRITE_REQUEST;
#pragma pack()



//
//This struct is used when telling the driver to send response data
//BlockingReads allows the caller to configure the driver to block if
//there is no data or to return. Set to 1 to block reads and 0 if reads
//should not block
//If the caller wishes to override the default buffer size then set BufferSize
//to the new size.  Let BufferSize equal zero to use the driver default. 
//

typedef struct {
	ULONG	BlockingReads;			// 0 - reads return, 1 - reads block 
	ULONG	BufferSize;				// 0 - use default  > - override default
} I2C_RESPONSE_REQUEST, *PI2C_RESPONSE_REQUEST;



//
// Command: I2CWriteRead
// 
// Writes data to an I2C slave device in Master Transmit mode then reads
// data from same device in Master Receive mode.
// Uses fields of nt2unixioctl as follows:
// lpvInBuffer - pointer to struct of type i2cWriteRequest.
// cbInBuffer - size of struct i2cWriteRequest plus data size
// lpvOutBuffer - pointer to buffer area for received data
// cbOutBuffer - size of the buffer area (maximum number of bytes to read)
// lpcbBytesReturned - pointer to location that will contains actual bytes read.
//
//
// Command: I2CWantResponseData
//
// This command informs the driver that this application will receive response
// messages and single byte events.  When response data arrives at the driver,
// the data will be sent to the read queue of all applications that have sent
// this ioctl.  Only the command field of the ioctl is used for this ioctl.
// lpvInBuffer - pointer to struct of type I2C_RESPONSE_REQUEST.
// cbInBuffer - size of struct I2C_RESPONSE_REQUEST
 
//
// Driver Read Data Format:
// The driver sends the data received while in Slave Receive to applications
// via the read(2) system call.   This is normally response data from a
// command or a single byte event message.  The driver maintains message
// boundaries; that is if one byte is received from one message and 100 bytes
// in another, the first read will receive the one byte message and a second
// read the 100 byte message.

/*M*
//	I2C_OK - command completed successfully.
//	I2C_INVALID_BUS - bad or incorrect bus number field.
//	I2C_INVALID_LENGTH - BytesToRead was not greater 0 and less than 1025.
//	I2C_BAD_ADDRESS - no I2C device responded to the supplied address
//	I2C_BUS_HUNG - the I2C driver never successfully arbitrated for the I2C
//			bus.  Probably a hardware problem.
//	I2C_NO_DATA - when using the I2C_NO_WAIT option, this status indicates
//	the requested data is not currently available.  This status should
//	only be returned when using I2C_NO_WAIT.
//	specified type, I2C_NO_EVENTS is returned. 
*M*/

#define I2C_OK			0
#define I2C_INVALID_BUS		1
#define I2C_INVALID_LENGTH	2
#define I2C_BAD_ADDRESS		3
#define I2C_BUS_HUNG		4
#define I2C_NO_DATA		5
#define I2C_NO_EVENTS		6

#define MAX_SLAVE_BUFFER_SIZE			1024


#define NUMBER_DEVICE_NODES             L"I2C Device Nodes"
#define RESPONSE_BUFFER_SIZE		L"Response Buffer Size"
#define	POLL_TIME			L"Poll Time"
#define	I2C_TIMEOUT			L"I2C Timeout"
#define DEFAULT_NUMBER_DEVICE_NODES     3
#define MAX_NODES						99		// do not change unless you increase buffer

#define DEFAULT_RESPONSE_BUFFER_SIZE	1024

#define DEFAULT_POLL_TIME				100000				// Poll every 1/10 second

#define DEFAULT_I2C_TIMEOUT				100000				// 100 milliseconds timeout


//
// Masks use to change settings
//

#define I2C_CHANGE_NUMBER_DEVICE_NODES_MASK		0x1
#define I2C_CHANGE_RESPONSE_BUFFER_SIZE_MASK	0x2
#define I2C_CHANGE_POLL_TIME_MASK				0x4
#define I2C_CHANGE_I2C_TIMEOUT_MASK				0x8


#define I2C_CHANGE_MASKS	(I2C_CHANGE_NUMBER_DEVICE_NODES_MASK | \
							I2C_CHANGE_RESPONSE_BUFFER_SIZE_MASK | \
							I2C_CHANGE_POLL_TIME_MASK | \
							I2C_CHANGE_I2C_TIMEOUT_MASK)	


typedef struct _I2C_PARAMETERS {
	ULONG	ChangeMask;				// each bits indicates the field to set
    ULONG   NumberOfDeviceNodes;
	ULONG	ResponseBufferSize;
	ULONG	PollTime;				// in Microseconds
	ULONG	I2CTimeout;				// in Microseconds
} I2C_PARAMETERS, *PI2C_PARAMETERS;




#endif // _I2CDRV_H













